SystemOrganization addCategory: #IntervalTree!

Object subclass: #IntervalTree
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTree commentStamp: 'rvc 3/3/2013 18:15' prior: 0!
An IntervalTree is an immutable interval collection stored in space proportional to O(n) for n intervals.  An interval tree answers the queries "which intervals in the tree intersect a given interval" and "which intervals in the tree contain a given value" in time proportional to O(log n) + k for answers containing k intervals.

Instance Variables
	root:		<Object>

root
	- the interval tree's root.
	
Invariant
      - root !!= null
!

!IntervalTree class methodsFor: 'instance creation' stamp: 'rvc 2/3/2013 13:34'!
withIntervals: intervalCollection

  "Answer an interval tree containing the given intervals."

  ^ super new initialize: (IntervalTreeIntervalBag withIntervals: intervalCollection)! !

!IntervalTree methodsFor: 'private initializing' stamp: 'rvc 2/5/2013 07:27'!
initialize: intervalBag

  "Initialize this interval tree to contain the given intervals.
   Answer this tree."

    root :=
      intervalBag empty
        ifTrue: [ IntervalTreeEmptyNode new ]
        ifFalse: [ IntervalTreeNonemptyNode withIntervals: intervalBag ]

  . ^ self! !

!IntervalTree methodsFor: 'querying' stamp: 'rvc 3/3/2013 15:58'!
intervalsContaining: aValue

  "Answer a plain old mutable bag of intervals in this tree
   containing the given value."

  ^ root intervalsIntersecting: (IntervalTreeInterval from: aValue to: aValue)! !

!IntervalTree methodsFor: 'querying' stamp: 'rvc 3/3/2013 15:58'!
intervalsIntersecting: anInterval

  "Answer a plain old mutable bag of intervals in this tree
   intersecting the given interval."

  ^ root intervalsIntersecting: anInterval! !

Object subclass: #IntervalTreeEmptyNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTreeEmptyNode commentStamp: 'rvc 3/3/2013 18:16' prior: 0!
An IntervalTreeEmptyNode is a interval-tree node containing no intervals.

Instance Variables
!

!IntervalTreeEmptyNode methodsFor: 'querying' stamp: 'rvc 3/17/2013 20:05'!
intervalsIntersecting: anInterval

  "Answer a plain old mutable bag of intervals in this tree intersecting the given interval."

  ^ Bag new! !

Object subclass: #IntervalTreeInterval
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTreeInterval commentStamp: 'rvc 3/5/2013 10:11' prior: 0!
An IntervalTreeInterval is an immutable interval [l, r] with l <= r.  l and r are objects that respond to the tricotomous relation <.

Instance Variables
  left  The interval's left end-point.
  right  The interval's right end-point

Invariants
  not right < left!

!IntervalTreeInterval class methodsFor: 'instance creation' stamp: 'rvc 1/31/2013 09:45'!
from: aValue to: anotherValue

  "Answer an interval having the given values."

  ^ super new initializeWith: aValue and: anotherValue.! !

!IntervalTreeInterval methodsFor: 'private initializing' stamp: 'rvc 1/31/2013 09:56'!
initializeWith: aValue and: anotherValue

  "Initialize this interval with the given end-point values.
   Answer this interval."

    aValue < anotherValue
      ifTrue: [ left := aValue . right := anotherValue ]
      ifFalse: [ left := anotherValue . right := aValue ]

  . ^ self! !

!IntervalTreeInterval methodsFor: 'querying' stamp: 'rvc 1/31/2013 09:48'!
left

  "Answer this interval's left end-point."

  ^ left! !

!IntervalTreeInterval methodsFor: 'querying' stamp: 'rvc 1/31/2013 09:48'!
right

  "Answer this interval's right end-point."

  ^ right! !

Object subclass: #IntervalTreeIntervalBag
	instanceVariableNames: 'intervals median'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTreeIntervalBag commentStamp: 'rvc 3/5/2013 10:13' prior: 0!
An IntervalTreeIntervalBag is an immutable bag of intervals.

Instance Variables

  intervlas: a plain old mutable bag of intervals.

  median: the median end-point value for the intervals in this bag.  
    Undefined if the bag is empty. !

!IntervalTreeIntervalBag class methodsFor: 'instance creation' stamp: 'rvc 2/3/2013 13:33'!
withIntervals: intervals

  "Answer an interval bag containing the given intervals"

  intervals class == IntervalTreeIntervalBag
    ifTrue: [ ^ intervals ]
    ifFalse: [ ^ self new initialize: intervals ]! !

!IntervalTreeIntervalBag methodsFor: 'converting' stamp: 'rvc 3/5/2013 13:50'!
asBag

  "Answer a plain old mutable bag containing the intervals in this interval bag."

  ^ intervals asBag! !

!IntervalTreeIntervalBag methodsFor: 'querying' stamp: 'rvc 2/3/2013 10:53'!
empty

  "Answer true iff this interval bag contains no intervals."

  ^ intervals size == 0! !

!IntervalTreeIntervalBag methodsFor: 'querying' stamp: 'rvc 2/3/2013 21:04'!
endpoints

  "Answer an array containing the endpoints of the intervals in this bag.
   The endpoints are stored in no particular order."

  | endpoints j |

    endpoints := Array new: (intervals size)*2
  . j := 1
  . intervals do: [ :i |
	  endpoints at: j put: i left
	. endpoints at: j + 1 put: i right
	. j := j + 2
	]

  . ^ endpoints! !

!IntervalTreeIntervalBag methodsFor: 'private initializing' stamp: 'rvc 3/6/2013 18:01'!
initialize: intervalCollection

  "Initialize this interval bag with the given intervals.
   Answer this bag."

  | j |

    intervals := Array new: (intervalCollection size)

  . j := 1
  . intervalCollection do: [ :i | intervals at: j put: i . j := j + 1 ]

  . ^self! !

!IntervalTreeIntervalBag methodsFor: 'querying' stamp: 'rvc 2/3/2013 20:57'!
median

  "Answer the median end-point value."

  | endpoints n |

    endpoints := self endpoints sort
  . n := (endpoints size)//2
  . ^ ((endpoints at: n) + (endpoints at: n + 1))/2! !

!IntervalTreeIntervalBag methodsFor: 'querying' stamp: 'rvc 3/5/2013 10:28'!
partitionBy: aValue

  "Answer a partition of the intervals in this bag by the given value.
   The answer is a three-elelemt array of interval bags.  Element 1
   contains intervals to the left of the given value (right < aValue);
   element 2 contains intervals straddling the given value
   (left <= aValue <= right) and element 3 contains intervals to the
    right of the given value (aValue < left)."

  | p n |

    n := 3
  . p := Array new: n
  . 1 to: n do: [ :i | p at: i put: (Bag new) ]

  . intervals do: [ :i |
	| j |
	  j := 2
	. i right < aValue
	    ifTrue: [ j := 1 ]
	    ifFalse: [ 
		aValue < i left
		  ifTrue: [ j := 3 ]
		]
	. (p at: j) add: i
	]

  . p replace: [ :b | IntervalTreeIntervalBag withIntervals: b ]
  . ^ p! !

Object subclass: #IntervalTreeNonemptyNode
	instanceVariableNames: 'median left right straddlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTreeNonemptyNode commentStamp: 'rvc 3/5/2013 10:15' prior: 0!
An IntervalTreeNonemptyNode is an interval-tree node containing at least one interval.
T is the interval tree rooted at this node.

Instance Variables
	left:		<Object>
	median:		<Object>
	right:		<Object>
	straddlers:		<IntervalTreeStraddlingIntervals>

left
	- This node's left child.

median
	- The median end-point value of all intervals in T.

right
	- This node's right child.

straddlers
	- The intervals in T containing the median.
!

!IntervalTreeNonemptyNode class methodsFor: 'instance creation' stamp: 'rvc 2/1/2013 00:18'!
withIntervals: intervals

  "Answer an interval tree containing the given intervals."

  ^ super new initialize: intervals! !

!IntervalTreeNonemptyNode methodsFor: 'private initializing' stamp: 'rvc 2/5/2013 20:16'!
initialize: intervalBag

  "Initialize this interval-tree node with the given intervals.
   Answer this node."

  | p |

    median := intervalBag median
  . p := intervalBag partitionBy: median
  . left := IntervalTree withIntervals: (p at: 1)
  . straddlers := IntervalTreeStraddlingIntervals withIntervals: (p at: 2)
  . right := IntervalTree withIntervals: (p at: 3)

  . ^ self! !

!IntervalTreeNonemptyNode methodsFor: 'querying' stamp: 'rvc 3/7/2013 10:24'!
intervalsIntersecting: anInterval

  "Answer a plain old mutable bag containing the intervals in this tree
   intersecting the given interval."

  . anInterval right < median
      ifTrue: [
	  ^ (left intervalsIntersecting: anInterval) 
	          addAll: (straddlers leftIntersectionsWith: anInterval) 
	        ; yourself ]

  . median < anInterval left 
      ifTrue: [
	  ^ (right intervalsIntersecting: anInterval) 
	          addAll: (straddlers rightIntersectionsWith: anInterval) 
	        ; yourself ]

  . ^ (straddlers intervals)  
           addAll: (left intervalsIntersecting: anInterval) 
         ; addAll: (right intervalsIntersecting: anInterval)
         ; yourself! !

Object subclass: #IntervalTreeStraddlingIntervals
	instanceVariableNames: 'intervals increasingLeftEndpoints decreasingRightEndpoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTreeStraddlingIntervals commentStamp: 'rvc 3/5/2013 10:18' prior: 0!
An IntervalTreeStraddlingIntervals is an immutable bag of straddling intervals for an interval-tree node.  A node n's straddling intervals is the subset of intervals in the tree T rooted at n containing the median of .the interval endpoints in T.

Instance Variables
	intervals:					<Bag>
      increasingLeftEndpionts:	<Array>
	decreasingRightEndpoints:	<Array>

  intervals:  the plain old mutable bag of intervals containing (straddling) the median
    end-point value for the intervals contained in the tree rooted at this node.

  increasingLeftEndpoints:  the straddling intervals ordered by increasing left 
    end-point value.

  decreasingRightEndpoints:  the straddling intervals ordered by decreasing
    right end-point value.

Invariants:
      intervals = permutation(increasingLeftEndpoints)

      intervals = permutation(decreasingRightEndpoints)

      1 <= i <= j <= size(intervals) implies 
        (decreasingRightEndpoints at: i) right >= (decreasingRightEndpoints at: j) right

      1 <= i <= j <= size(intervals) implies 
        (increasingLeftEndpoints at: i) left <= (increasingLeftEndpoints at: j) left
!

!IntervalTreeStraddlingIntervals class methodsFor: 'instance creation' stamp: 'rvc 2/15/2013 13:40'!
withIntervals: anIntervalBag

  "Answer a straddling-intervals bag containing the straddling intervals in the given bag."

  ^ super new initialize: anIntervalBag! !

!IntervalTreeStraddlingIntervals methodsFor: 'private initializing' stamp: 'rvc 3/17/2013 19:53'!
initialize: anIntervalBag

  "Initialize this straddling-interval collection from the given intervals.
   Answer this collection."

    intervals := anIntervalBag asBag
  . increasingLeftEndpoints := intervals asArray sort: [ :a :b | a left < b left ] ; yourself
  . decreasingRightEndpoints := intervals asArray sort: [ :a :b | b right <  a right ] ; yourself

  . ^ self! !

!IntervalTreeStraddlingIntervals methodsFor: 'querying' stamp: 'rvc 3/3/2013 11:18'!
intervals

  "Answer a plain old mutable bag of straddling intervals."

  ^ intervals copy! !

!IntervalTreeStraddlingIntervals methodsFor: 'querying' stamp: 'rvc 3/3/2013 11:19'!
leftIntersectionsWith: anInterval

  "Answer a plain old mutable bag of intervals from this node that intersect
   the given interval.  The given interval is assumed to be to the left of
   the median end-point value (anInterval right < m) for this subtree's intervals."

  | b r |

    b := Bag new: intervals size
  . r := anInterval right

  . increasingLeftEndpoints do: [ :i |
	  r < i left ifTrue: [ ^ b ]
	. b add: i
	]

  . ^ b! !

!IntervalTreeStraddlingIntervals methodsFor: 'querying' stamp: 'rvc 3/3/2013 11:19'!
rightIntersectionsWith: anInterval

  "Answer a plain old mutable bag of intervals in this node that intersect
   the given interval.  The given interval is assumed to be to the right of
   the median end-point value (m < anInterval left) for this subtree's intervals."

  | b l |

    b := Bag new: intervals size
  . l := anInterval left

  . decreasingRightEndpoints do: [ :i |
	  i right < l ifTrue: [ ^ b ]
	. b add: i
	]

  . ^ b! !

TestCase subclass: #IntervalTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'IntervalTree'!

!IntervalTreeTests commentStamp: 'rvc 3/17/2013 22:20' prior: 0!
An IntervalTreeTests is a bunch of tests for interval tree components.

Instance Variables
!

!IntervalTreeTests methodsFor: 'interval bag tests' stamp: 'rvc 2/3/2013 13:38'!
testBagCreation

  | b1 b2 i |

    b1 := IntervalTreeIntervalBag withIntervals: #()
  . self assert: (IntervalTreeIntervalBag withIntervals: b1) == b1

  . i := Set with: (IntervalTreeInterval from: 1 to: 3)
  . b2 := IntervalTreeIntervalBag withIntervals: i
  . self deny: b1 == i
  . self deny: b1 == b2

  
! !

!IntervalTreeTests methodsFor: 'interval bag tests' stamp: 'rvc 2/3/2013 13:34'!
testBagEmpty

  | i |

    i := Set new
  . self assert: (IntervalTreeIntervalBag withIntervals: i) empty
  . i add: (IntervalTreeInterval from: 1 to: 3)
  . self deny: (IntervalTreeIntervalBag withIntervals: i) empty
! !

!IntervalTreeTests methodsFor: 'interval bag tests' stamp: 'rvc 3/5/2013 13:47'!
testBagPartition

  | i b p |

    i := IntervalTreeInterval from: 1 to: 3
  . b := IntervalTreeIntervalBag withIntervals: (Set with: i)

  . p := b partitionBy: 0
  . self assert: (p at: 1) empty
  . self assert: (p at: 2) empty
  . self deny: (p at: 3) empty

  . p := b partitionBy: 2
  . self assert: (p at: 1) empty
  . self assert: (p at: 3) empty
  . self deny: (p at: 2) empty

  . p := b partitionBy: 4
  . self assert: (p at: 3) empty
  . self assert: (p at: 2) empty
  . self deny: (p at: 1) empty

  . b := IntervalTreeIntervalBag withIntervals: (
      Set with: i 
        with: (IntervalTreeInterval from: -3 to: -1))

  . p := b partitionBy: 0
  . self deny: (p at: 1) empty
  . self assert: (p at: 2) empty
  . self deny: (p at: 3) empty

  . b := IntervalTreeIntervalBag withIntervals: (
      Set with: i 
        with: (IntervalTreeInterval from: -3 to: -1)
        with: (IntervalTreeInterval from: 5 to: 7))

  . p := b partitionBy: 2
  . self deny: (p at: 1) empty
  . self deny: (p at: 2) empty
  . self deny: (p at: 3) empty

  . b := IntervalTreeIntervalBag withIntervals: (Bag with: i with: i)

  . p := b partitionBy: 0
  . self assert: (p at: 1) empty
  . self assert: (p at: 2) empty
  . self deny: (p at: 3) empty

  . p := b partitionBy: 0
  . self assert: (p at: 1) empty
  . self assert: (p at: 2) empty
  . self assert: (p at: 3) asBag size = 2

  . p := b partitionBy: 2
  . self assert: (p at: 1) empty
  . self assert: (p at: 2) asBag size = 2
  . self assert: (p at: 3) empty

  . p := b partitionBy: 4
  . self assert: (p at: 1) asBag size = 2
  . self assert: (p at: 2) empty
  . self assert: (p at: 3) empty



! !

!IntervalTreeTests methodsFor: 'empty node tests' stamp: 'rvc 1/31/2013 20:55'!
testEmptyNode

  | n |

    n := IntervalTreeEmptyNode new
  . self assert: (n intervalsIntersecting: (IntervalTreeInterval from: 1 to: 2)) size == 0! !

!IntervalTreeTests methodsFor: 'interval tree tests' stamp: 'rvc 1/31/2013 21:03'!
testEmptyTree

  | t |

    t := IntervalTree withIntervals: #()
  . self assert: (t intervalsIntersecting: (IntervalTreeInterval from: 1 to: 2)) size == 0
  . self assert: (t intervalsContaining: 0) size == 0! !

!IntervalTreeTests methodsFor: 'interval bag tests' stamp: 'rvc 2/3/2013 21:10'!
testEndpoints

  | i b p |

    i := IntervalTreeInterval from: 1 to: 3
  . b := IntervalTreeIntervalBag withIntervals: (Set with: i)
  . p := b endpoints
  . self assert: p size == 2
  . self assert: ((p literalEqual: #( 1 3 )) or: (p literalEqual: #( 3 1 )))! !

!IntervalTreeTests methodsFor: 'interval tests' stamp: 'rvc 1/31/2013 17:58'!
testFromTo

  | i |

    i := IntervalTreeInterval from: 1 to: 2
  . self assert: i left == 1
  . self assert: i right == 2

  . i := IntervalTreeInterval from: 2 to: 1
  . self assert: i left == 1
  . self assert: i right == 2
! !

!IntervalTreeTests methodsFor: 'interval tree tests' stamp: 'rvc 3/17/2013 22:25'!
testGappyTriangle

  self testTriangleOfSize: 9 withGapSize: 0.1! !

!IntervalTreeTests methodsFor: 'interval bag tests' stamp: 'rvc 2/3/2013 13:34'!
testIntervalBag

  | b |

    b := IntervalTreeIntervalBag withIntervals: #()
! !

!IntervalTreeTests methodsFor: 'straddling interval tests' stamp: 'rvc 3/2/2013 18:36'!
testLeftIntersection

  | i b b2 |

    i := IntervalTreeInterval from: 1 to: 3
  . b := IntervalTreeIntervalBag withIntervals: (Bag with: i)
  . b := IntervalTreeStraddlingIntervals withIntervals: b
  . b2 := b leftIntersectionsWith: i
  . self assert: b2 size = 1
  . self assert: b2 anyOne = i

  . b2 := b leftIntersectionsWith: (IntervalTreeInterval from: 1.5 to: 2.5)
  . self assert: b2 size = 1
  . self assert: b2 anyOne = i

  . i := IntervalTreeInterval from: -1 to: 0
  . b2 := b leftIntersectionsWith: i
  . self assert: b2 size = 0! !

!IntervalTreeTests methodsFor: 'straddling interval tests' stamp: 'rvc 3/17/2013 20:03'!
testLeftIntersection2

  | i il im ir b b2 |

    il := IntervalTreeInterval from: 1 to: 2
  . im := IntervalTreeInterval from: 1.3 to: 2.3
  . ir := IntervalTreeInterval from: 1.6 to: 2.6
  . b := (Bag new: 3) add: il ; add: im ; add: ir ; yourself
  . b := IntervalTreeIntervalBag withIntervals: b
  . b := IntervalTreeStraddlingIntervals withIntervals: b

  . i := IntervalTreeInterval from: 1.7 to: 1.7
  . b2 := b leftIntersectionsWith: i
  . self assert: b2 size = 3
  . self assert: (b2 includes: ir)
  . self assert: (b2 includes: im)
  . self assert: (b2 includes: il)

  . i := IntervalTreeInterval from: 1.4 to: 1.4
  . b2 := b leftIntersectionsWith: i
  . self assert: b2 size = 2
  . self assert: (b2 includes: il)
  . self assert: (b2 includes: im)

  . i := IntervalTreeInterval from: 1.1 to: 1.1
  . b2 := b leftIntersectionsWith: i
  . self assert: b2 size = 1
  . self assert: (b2 includes: il)
! !

!IntervalTreeTests methodsFor: 'interval bag tests' stamp: 'rvc 2/3/2013 21:21'!
testMedian

  | b i |

    i := Set with: (IntervalTreeInterval from: 1 to: 3)
  . b := IntervalTreeIntervalBag withIntervals: i
  . self assert: b median = 2

  . i add: (IntervalTreeInterval from: 4 to: 5)
  . b := IntervalTreeIntervalBag withIntervals: i
  . self assert: b median = 3.5! !

!IntervalTreeTests methodsFor: 'straddling interval tests' stamp: 'rvc 3/3/2013 11:01'!
testRightIntersection

  | i b b2 |

    i := IntervalTreeInterval from: 1 to: 3
  . b := IntervalTreeIntervalBag withIntervals: (Bag with: i)
  . b := IntervalTreeStraddlingIntervals withIntervals: b
  . b2 := b rightIntersectionsWith: i
  . self assert: b2 size = 1
  . self assert: b2 anyOne = i

  . b2 := b rightIntersectionsWith: (IntervalTreeInterval from: 1.5 to: 2.5)
  . self assert: b2 size = 1
  . self assert: b2 anyOne = i

  . i := IntervalTreeInterval from: 4 to: 5
  . b2 := b rightIntersectionsWith: i
  . self assert: b2 size = 0! !

!IntervalTreeTests methodsFor: 'straddling interval tests' stamp: 'rvc 3/17/2013 19:58'!
testRightIntersection2

  | i il im ir b b2 |

    il := IntervalTreeInterval from: 1 to: 2
  . im := IntervalTreeInterval from: 1.3 to: 2.3
  . ir := IntervalTreeInterval from: 1.6 to: 2.6
  . b := (Bag new: 3) add: il ; add: im ; add: ir ; yourself
  . b := IntervalTreeIntervalBag withIntervals: b
  . b := IntervalTreeStraddlingIntervals withIntervals: b

  . i := IntervalTreeInterval from: 1.9 to: 1.9
  . b2 := b rightIntersectionsWith: i
  . self assert: b2 size = 3
  . self assert: (b2 includes: ir)
  . self assert: (b2 includes: im)
  . self assert: (b2 includes: il)

  . i := IntervalTreeInterval from: 2.2 to: 2.2
  . b2 := b rightIntersectionsWith: i
  . self assert: b2 size = 2
  . self assert: (b2 includes: ir)
  . self assert: (b2 includes: im)

  . i := IntervalTreeInterval from: 2.4 to: 2.4
  . b2 := b rightIntersectionsWith: i
  . self assert: b2 size = 1
  . self assert: (b2 includes: ir)
! !

!IntervalTreeTests methodsFor: 'straddling interval tests' stamp: 'rvc 2/15/2013 13:45'!
testStraddlingIntervals

  | i |

    i := IntervalTreeInterval from: 1 to: 3
  . i := Bag with: i
  . i := IntervalTreeIntervalBag withIntervals: i
  . i := IntervalTreeStraddlingIntervals withIntervals: i! !

!IntervalTreeTests methodsFor: 'interval tree tests' stamp: 'rvc 2/4/2013 10:30'!
testTreeCreate

  | t i |

    i := IntervalTreeInterval from: 1 to: 2
  . t := IntervalTree withIntervals: (Set with: i)
  . self assert: (t intervalsIntersecting: i) size = 1
  . self assert: (t intervalsContaining: 1) size = 1! !

!IntervalTreeTests methodsFor: 'utilities' stamp: 'rvc 3/17/2013 22:27'!
testTriangleOfSize: n withGapSize: gapSize

  | b t |

    b := Bag new
  . 1 to: n do: [ :i |
	| ivl |
	  ivl := IntervalTreeInterval from: i + gapSize to: i + 1
	. i timesRepeat: [ b add: ivl ]
	]
  . t := IntervalTree withIntervals: b
  . 0 to: n do: [ :i |
	  b := t intervalsContaining: i + 0.5
	. self assert: b size = i
	]! !

!IntervalTreeTests methodsFor: 'interval tree tests' stamp: 'rvc 3/16/2013 16:18'!
testTwoIntervalTree

  | t i |

    i := IntervalTreeInterval from: 1 to: 2
  . t := IntervalTree withIntervals: (Bag with: i with: i with: (IntervalTreeInterval from: 3 to: 4))
  . self assert: (t intervalsContaining: 1.4) size = 2
! !

!IntervalTreeTests methodsFor: 'interval tree tests' stamp: 'rvc 3/17/2013 22:24'!
testUngappyTriangle

  self testTriangleOfSize: 9 withGapSize: 0! !
